"""add icon field to plugins

Revision ID: 35ebb38695cd
Revises: 9a9df20ea5a7
Create Date: 2025-12-24 11:39:42.229109

"""

from alembic import op
import sqlalchemy as sa
import sqlalchemy_utils


# revision identifiers, used by Alembic.
revision = "35ebb38695cd"
down_revision = "9a9df20ea5a7"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("plugin", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column("icon", sa.String(length=255), nullable=True)
        )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("plugin", schema=None) as batch_op:
        batch_op.drop_column("icon")

    # ### end Alembic commands ###
