"""add position to metadata descriptor

Revision ID: a1b2c3d4e5f6
Revises: 5f1620d191af
Create Date: 2025-11-01 12:00:00.000000

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "a1b2c3d4e5f6"
down_revision = "5f1620d191af"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "metadata_descriptor",
        sa.Column("position", sa.Integer(), nullable=True),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("metadata_descriptor", "position")
    # ### end Alembic commands ###
